%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Main program to generate the simulations
%%%%
%%%% INPUT: (1) number of repetitions
%%%%        (2) mean ON/OFF duration
%%%% STEPS
%%%% 1. Generate the simulated trace using exponential ON and OFF duration
%%%% 2. Call SAIL to get the parameters and the variance
%%%% 3. Validate
%%%% OUTPUT:(1) The loss rate
%%%%        (2) The variance
%%%%        (3) The 95% confidence interval
%%%%        (4) The mean ON/OFF durations
%%%% COPYRIGHT : Hung X. Nguyen, the University of Adelaide, Australia
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all

disp('Preparing the paths for function calls...');
addpath LIBRARY


ONP=1/10; %%%% 1/ average length of the Lossy period
OFFP =1/990; %%%1/average length of the Lossless period
thinningS=2;
lambda = 0.1;


%%%gamma dist parameters
ko=[5 1];
thetao=[1/(ko(1)*ONP)  1/(ko(2)*OFFP)];
noRep=1;


kArray=[];
thetaArray=[];
stationaryArray=[];
renewalArray=[];
LRINFER=[];
LREMP=[];
LRVAL=[];
VARINFER=[];
VAREMP=[];
VARVAL=[];
VARIND=[];
CONF=[];

lambdaArray=[0.4:0.1:0.4];
k1Array=[1:1:10];
k2Array=[1:1:10];

for lambdaID =1:1:1 
    %theta(2)=thetaidx;
    for k1ID=3:2:3
        ko(1)=k1Array(k1ID);
        for k2ID=1:2:1
            ko(2)=k2Array(k2ID);           
            for rep=1:noRep
                
                lambda=lambdaArray(lambdaID);  
                
                clear trce A_seq R_seq trce trceTime ON_seq OFF_seq k theta VEST
                
                thetao=[1/(ko(1)*ONP)  1/(ko(2)*OFFP)];
                n_samples=thinningS*min(10000,floor(500*(1/OFFP +1/ONP)*lambda));
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%%%% Simulate the input
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                disp(['Simulating the Renewal Process with parameters:',...
                    ' lambda=',num2str(lambda/thinningS),' ,k(1)=',num2str(ko(1)),' ,k(2)=',...
                     num2str(ko(2)),' ,theta(1)=',num2str(thetao(1)),' ,theta(2)=',num2str(thetao(2))]);                

                %%%%%% Gamma
                trce=-1*ones(n_samples,1);
                A_seq = cumsum(- log(rand(n_samples,1))/lambda); %% arrival time of samples;
                ON_seq= mygamrnd(ko(1),thetao(1),2*n_samples,0);
                OFF_seq= mygamrnd(ko(2),thetao(2),2*n_samples,0);
                
                R_seq=[0];
                
                idx=1;
                for i=1:2*n_samples
                    R_seq(idx+1) =R_seq(idx) + ON_seq(i);
                    R_seq(idx+2)= R_seq(idx+1) + OFF_seq(i);
                    idx=idx+2;
                end
                
                current_j=1;
                for i=1:n_samples
                    for j=current_j:length(R_seq)-1
                        if ((A_seq(i) > R_seq(j))&&(A_seq(i)<=R_seq(j+1)))
                            current_j=j;
                            break;
                        end
                    end
                    
                    if (mod(current_j,2) == 1)
                        trce(i)=0; %loss
                    else
                        trce(i)=1; %good
                    end
                    
                end
                
                %Checking for stationarity
                % Simulated traces should pass this test
                stationary=isStationary(trce);
                renewal=isRenewal(trce);
                
          
                %%%% divide the trace into two
                inferTrce =[];
                validTrce =[];
                

                %tempk1=[1:1:ceil(length(trce)/thinningS)];
                %tempk2=[(ceil(length(trce)/thinningS)+1):1:length(trce)];
                tempk1=[1:thinningS:length(trce)];
                tempk2=[2:thinningS:length(trce)];
                
                inferTrce= trce(tempk1);
                validTrce = trce(tempk2);
                trceTime=A_seq(tempk1);
                lambda=lambda/thinningS;
                n_samples = length(inferTrce);
                %%%%%%%%%End data generation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % Calling SAIL
                disp('Calling SAIL ...');
                tic
                [k theta VEST]=SAIL(inferTrce,trceTime);
                toc
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %Compute confidence intervals
                disp('Computing the confidence interval ..');
                intType =1 ; %1 is Guassian, 2 is Wilson, 3 is Jeffreys
                [upper lower]=getConfidenceInterval(inferTrce,theta,k,VEST,intType);
         
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%% Collecting statistics
                disp('Collecting statistics ...');
                kArray=[kArray;k];
                thetaArray=[thetaArray;theta];
                stationaryArray=[stationaryArray stationary];
                renewalArray=[renewalArray renewal];
                
                LRVAL=[LRVAL mean(1-validTrce)];
                LRINFER=[LRINFER theta(1)*k(1)/sum(theta.*k)];
                LREMP=[LREMP mean(1-inferTrce)];
                
                CONF=[CONF; [upper lower]];
                
                
                VIND=(mean(inferTrce)*(1-mean(inferTrce))/length(inferTrce));
                VARIND=[VARIND VIND];
                
                VARINFER=[VARINFER VEST];
                
                c=autocov(inferTrce,0:1:length(inferTrce)-2);
                c=c(find(~isnan(c)));
                VAREMP=[VAREMP (var(inferTrce) + 2*sum(c(2:length(c))'.*(1-[1:(length(c)-1)]/length(inferTrce))))/length(inferTrce)];
                
                c=autocov(validTrce,0:1:length(validTrce)-2);
                c=c(find(~isnan(c)));
                VARVAL=[VARVAL (var(validTrce) +2*sum(c(2:length(c))'.*(1-[1:(length(c)-1)]/length(validTrce))))/length(inferTrce)];
            end
        end
        
    end %rep
end %lambda
disp('k(1) k(2)');
disp(kArray);
disp('theta(1) theta(2)');
disp(thetaArray);
disp('Loss Array');
disp(LRVAL);
disp(LRINFER);
disp(LREMP);
disp('Variance');
disp(VARINFER);
disp('Confidence Interval (upper lower)');
disp(CONF);
